import java.util.*;

    /* A class to format lines in `ragged-right' format.
     * Words are added one-by-one to a line of a particular length.
     * Consecutive words should be separated by a single space.
     * If adding a word would cause a line to exceed its given
     * length then the current line must be printed and a new one
     * started. The (possibly incomplete) contents of a line may be
     * `flushed'.
     */
class LineFill {
        // Specify the maximum width of the line.
    public LineFill(int width){
        setWidth(width);
    }

        // Add the word to the current line if there is room.
        // Otherwise print the line and use word to start a new one.
    public void addWord(String word){
        // Words are retained in an ArrayList until a line is
        // full. The amount of space occupied by the words, plus
        // each single inter-word space is stored in the occupied
        // attribute.
        final int wordLength = word.length();
        final ArrayList words = getWords();
        int occupied = getOccupied();

        // See if a separating space will needed.
        if(occupied == 0){
            // Start of line.
            setOccupied(occupied+wordLength);
        }
        else{
            if((occupied+1+wordLength) <= getWidth()){
                // The word will fit.
                setOccupied(occupied+1+wordLength);
            }
            else{
                // Overflow. Line fill the existing contents.
                fillLine();
                setOccupied(wordLength);
            }
        }
        words.add(word);
    }

        // Print the current line and clear it ready for the next.
        // Do not fill the line.
    public void flush(){
        final ArrayList words = getWords();
        final int numWords = words.size();
        for(int i = 0; i < numWords-1; i++){
            System.out.print((String) words.get(i));
            System.out.print(' ');
        }
        // Print the final word on the line.
        System.out.println((String) words.get(numWords-1));
        words.clear();
        setOccupied(0);
    }

    public int getWidth(){
        return width;
    }

    public void setWidth(int w){
        if(w > 0){
            width = w;
        }
    }

        // Distribute spaces as evenly as possible between words.
        // Any excess will be distributed between
        // as many words as possible from left to right.
    private void fillLine(){
        final ArrayList words = getWords();
        final int numWords = words.size();
        if(numWords > 1){

            // Calculate the number of spare spaces, in excess of
            // a single space per word.
            final int spareSpaces = getWidth()-getOccupied();

            // Calculate how many extra spaces every inter-word gap
            // receives. Add in the standard single space that has
            // already been catered for.
            final int evenSpaces = 1+spareSpaces / (numWords-1);
            // Create a string that is evenSpaces long.
            final StringBuffer spaceBuffer = new StringBuffer(evenSpaces);
            for(int j = 0; j < evenSpaces; j++){
                 spaceBuffer.append(' ');
            }
            final String commonGap = spaceBuffer.toString();

            // Calculate how many inter-word gaps will have a further
            // space. This will be decremented by 1 after each use.
            int unevenSpaces = spareSpaces % (numWords-1);

            // Build up the line to be printed.
            StringBuffer outputLine = new StringBuffer(getWidth());
            for(int i = 0; i < numWords-1; i++){
                outputLine.append((String) words.get(i));
                outputLine.append(commonGap);
                if(unevenSpaces > 0){
                     outputLine.append(' ');
                     unevenSpaces--;
                }
            }
            // Place the final word on the line. This should appear
            // up against the right margin.
            outputLine.append((String) words.get(numWords-1));
            System.out.println(outputLine.toString());
            words.clear();
            setOccupied(0);
        }
        else{
            flush();
        }
    }

    private ArrayList getWords(){
        return words;
    }

    private int getOccupied(){
        return occupied;
    }

    private void setOccupied(int o){
        occupied = o;
    }

    // The width of the line.
    private int width;
    // The words of the line are held in words, until formatted.
    private ArrayList words = new ArrayList();
    // The amount of space currently occupied by the words
    // plus a single space between each pair.
    private int occupied = 0;
}
