        // Provide game management functionality for a multi-player
        // card game. The number of players participating
        // is passed to dealTheCards.
        // Use one CautiousPlayer and PlayHigh players for the
        // remainder.
        // All players will implement the CardPlayer interface.
class GameManager {
    public static final int minimumNumPlayers = 2;

        // Create a new pack and distribute cards to the players.
    public void dealTheCards(int numPlayers,int cardsPerSuit){
        // Create the appropriate number of players for the game.
        setupPlayers(numPlayers);
        final CardPlayer[] players = getPlayers();

        // Distribute the cards fairly so that each
        // players has the same number of cards.
        // Discard any remainder.
        final Pack pack = new Pack(cardsPerSuit);
        while(pack.size() >= numPlayers){
            for(int playerId = 0; playerId < numPlayers; playerId++){
                SuitedCard card = pack.nextCard();
                players[playerId].takeCard(card);
            }
        }
        // Inform the players of the discarded cards.
        while(pack.size() > 0){
            for(int playerId = 0; playerId < numPlayers; playerId++){
                SuitedCard card = pack.nextCard();
                players[playerId].undealtCard(card);
            }
        }
    }

        // The cards have been distributed; supervise a game.
    public void superviseAGame(){
        final CardPlayer[] players = getPlayers();
        final int numPlayers = players.length;
        // Keep track of the winner of each round.
        final int[] scores = new int[numPlayers];
        // Where to show the game.
        GameDisplay display = new GameDisplay(numPlayers);
        display.clear();

        // Create a pile for the first round.
        Pile pile = new Pile();

        // Select the first player of the first round.
        CardPlayer player = players[0];

        // Play as long as the players have cards left.
        // At the start of each round, each player has the
        // same number of cards.
        for(int round = 1; player.numCards() > 0; round++){
            display.report("Round "+round);
            pile.clear();
            // Request a card of any suit.
            SuitedCard firstCard = player.playAnyCard();
            // Add it to the pile.
            pile.addCard(firstCard);
            display.showCard(firstCard,player.getIdentity());
            // Show the card to all the players.
            for(int i = 0; i < numPlayers; i++){
                players[i].viewCard(firstCard);
            }

            // The remaining players must follow suit.
            String leadSuit = firstCard.getSuit();
            // Keep track of who is currently leading the round.
            CardPlayer leadingPlayer = player;
            SuitedCard leadingCard = firstCard;

            // Request the remaining players to play.
            int playerId = player.getIdentity();
            while(pile.numCards() < numPlayers){
                playerId = (playerId+1) % numPlayers;
                player = players[playerId];
                SuitedCard card = player.followSuit(leadSuit);
                display.showCard(card,player.getIdentity());
                pile.addCard(card);
                // Show the card to all the players.
                for(int i = 0; i < numPlayers; i++){
                    players[i].viewCard(card);
                }

                // See if this is a better card of the original suit.
                if(card.getSuit().equals(leadSuit)){
                    if(card.getValue() > leadingCard.getValue()){
                        leadingCard = card;
                        leadingPlayer = player;
                    }
                }
            }

            // Credit the round winner.
            int roundWinner = leadingPlayer.getIdentity();
            display.report(roundWinner+" wins the round.");
            scores[roundWinner]++;
            // Inform the players who won.
            for(int i = 0; i < numPlayers; i++){
                players[i].roundWinner(roundWinner);
            }
            // The winner starts the next round.
            player = leadingPlayer;
        }

        // Print the end-of-game scores.
        display.clear();
        display.report("The End of Game Scores");
        for(int i = 0; i < numPlayers; i++){
            display.report("Player: "+i+" scored "+scores[i]);
        }
    }

    private void setupPlayers(int numPlayers){
        // Set up the game's players.
        // Force a minimum number of players.
        if(numPlayers < minimumNumPlayers){
            numPlayers = minimumNumPlayers;
        }
        this.players = new CardPlayer[numPlayers];
        // Make the first a CautiousPlayer, and the
        this.players[0] = new CautiousPlayer(0);
        // remainder as PlayHigh players.
        for(int i = 1; i < numPlayers; i++){
            // Pass the player their identifying number.
            this.players[i] = new PlayHigh(i);
        }
    }

    private CardPlayer[] getPlayers(){
        return players;
    }

    // The players are set up as a result of calling
    // deal the cards.
    private CardPlayer[] players = new CardPlayer[0];
}
